@@ -29,20 +29,13 @@
 #include <config.h>
 #endif
 
-#include <stdlib.h>
 #include <string.h>
-#include <gio/gio.h>
 
 #define LIBSOUP_USE_UNSTABLE_REQUEST_API
 
 #include "soup-cache.h"
 #include "soup-cache-private.h"
-#include "soup-date.h"
-#include "soup-enum-types.h"
-#include "soup-headers.h"
-#include "soup-session.h"
-#include "soup-session-feature.h"
-#include "soup-uri.h"
+#include "soup.h"
 
 static SoupSessionFeatureInterface *soup_cache_default_feature_interface;
 static void soup_cache_session_feature_init (SoupSessionFeatureInterface *feature_interface, gpointer interface_data);
@@ -108,6 +101,7 @@ typedef struct {
 	SoupCache *cache;
 	SoupCacheEntry *entry;
 	SoupMessage *msg;
+	gulong content_sniffed_handler;
 	gulong got_chunk_handler;
 	gulong got_body_handler;
 	gulong restarted_handler;
@@ -163,7 +157,7 @@ get_cacheability (SoupCache *cache, SoupMessage *msg)
 	if (content_type && !g_ascii_strcasecmp (content_type, "multipart/x-mixed-replace"))
 		return SOUP_CACHE_UNCACHEABLE;
 
-	cache_control = soup_message_headers_get (msg->response_headers, "Cache-Control");
+	cache_control = soup_message_headers_get_list (msg->response_headers, "Cache-Control");
 	if (cache_control && *cache_control) {
 		GHashTable *hash;
 		SoupCachePrivate *priv = SOUP_CACHE_GET_PRIVATE (cache);
@@ -274,25 +268,10 @@ soup_cache_entry_free (SoupCacheEntry *entry, GFile *file)
 	}
 
 	g_free (entry->uri);
-	entry->uri = NULL;
-
-	if (entry->current_writing_buffer) {
-		soup_buffer_free (entry->current_writing_buffer);
-		entry->current_writing_buffer = NULL;
-	}
-
-	if (entry->headers) {
-		soup_message_headers_free (entry->headers);
-		entry->headers = NULL;
-	}
-	if (entry->error) {
-		g_error_free (entry->error);
-		entry->error = NULL;
-	}
-	if (entry->cancellable) {
-		g_object_unref (entry->cancellable);
-		entry->cancellable = NULL;
-	}
+	g_clear_pointer (&entry->current_writing_buffer, soup_buffer_free);
+	g_clear_pointer (&entry->headers, soup_message_headers_free);
+	g_clear_error (&entry->error);
+	g_clear_object (&entry->cancellable);
 
 	g_slice_free (SoupCacheEntry, entry);
 }
@@ -345,7 +324,7 @@ soup_cache_entry_set_freshness (SoupCacheEntry *entry, SoupMessage *msg, SoupCac
 	const char *cache_control;
 	const char *expires, *date, *last_modified;
 
-	cache_control = soup_message_headers_get (entry->headers, "Cache-Control");
+	cache_control = soup_message_headers_get_list (entry->headers, "Cache-Control");
 	if (cache_control) {
 		const char *max_age, *s_maxage;
 		gint64 freshness_lifetime = 0;
@@ -388,8 +367,8 @@ soup_cache_entry_set_freshness (SoupCacheEntry *entry, SoupMessage *msg, SoupCac
 	/* If the 'Expires' response header is present, use its value
 	 * minus the value of the 'Date' response header
 	 */
-	expires = soup_message_headers_get (entry->headers, "Expires");
-	date = soup_message_headers_get (entry->headers, "Date");
+	expires = soup_message_headers_get_one (entry->headers, "Expires");
+	date = soup_message_headers_get_one (entry->headers, "Date");
 	if (expires && date) {
 		SoupDate *expires_d, *date_d;
 		time_t expires_t, date_t;
@@ -419,21 +398,21 @@ soup_cache_entry_set_freshness (SoupCacheEntry *entry, SoupMessage *msg, SoupCac
 
 	/* Otherwise an heuristic may be used */
 
-	/* Heuristics MUST NOT be used with these status codes
-	   (section 2.3.1.1) */
-	if (msg->status_code != SOUP_STATUS_OK &&
-	    msg->status_code != SOUP_STATUS_NON_AUTHORITATIVE &&
-	    msg->status_code != SOUP_STATUS_PARTIAL_CONTENT &&
-	    msg->status_code != SOUP_STATUS_MULTIPLE_CHOICES &&
-	    msg->status_code != SOUP_STATUS_MOVED_PERMANENTLY &&
-	    msg->status_code != SOUP_STATUS_GONE)
+	/* Heuristics MUST NOT be used with stored responses with
+	   these status codes (section 2.3.1.1) */
+	if (entry->status_code != SOUP_STATUS_OK &&
+	    entry->status_code != SOUP_STATUS_NON_AUTHORITATIVE &&
+	    entry->status_code != SOUP_STATUS_PARTIAL_CONTENT &&
+	    entry->status_code != SOUP_STATUS_MULTIPLE_CHOICES &&
+	    entry->status_code != SOUP_STATUS_MOVED_PERMANENTLY &&
+	    entry->status_code != SOUP_STATUS_GONE)
 		goto expire;
 
 	/* TODO: attach warning 113 if response's current_age is more
 	   than 24h (section 2.3.1.1) when using heuristics */
 
 	/* Last-Modified based heuristic */
-	last_modified = soup_message_headers_get (entry->headers, "Last-Modified");
+	last_modified = soup_message_headers_get_one (entry->headers, "Last-Modified");
 	if (last_modified) {
 		SoupDate *soup_date;
 		time_t now, last_modified_t;
@@ -482,7 +461,7 @@ soup_cache_entry_new (SoupCache *cache, SoupMessage *msg, time_t request_time, t
 	soup_cache_entry_set_freshness (entry, msg, cache);
 
 	/* Section 2.3.2, Calculating Age */
-	date = soup_message_headers_get (entry->headers, "Date");
+	date = soup_message_headers_get_one (entry->headers, "Date");
 
 	if (date) {
 		SoupDate *soup_date;
@@ -493,7 +472,7 @@ soup_cache_entry_new (SoupCache *cache, SoupMessage *msg, time_t request_time, t
 		date_value = soup_date_to_time_t (soup_date);
 		soup_date_free (soup_date);
 
-		age = soup_message_headers_get (entry->headers, "Age");
+		age = soup_message_headers_get_one (entry->headers, "Age");
 		if (age)
 			age_value = g_ascii_strtoll (age, NULL, 10);
 
@@ -515,6 +494,8 @@ soup_cache_writing_fixture_free (SoupCacheWritingFixture *fixture)
 	/* Free fixture. And disconnect signals, we don't want to
 	   listen to more SoupMessage events as we're finished with
 	   this resource */
+	if (g_signal_handler_is_connected (fixture->msg, fixture->content_sniffed_handler))
+		g_signal_handler_disconnect (fixture->msg, fixture->content_sniffed_handler);
 	if (g_signal_handler_is_connected (fixture->msg, fixture->got_chunk_handler))
 		g_signal_handler_disconnect (fixture->msg, fixture->got_chunk_handler);
 	if (g_signal_handler_is_connected (fixture->msg, fixture->got_body_handler))
@@ -528,6 +509,12 @@ soup_cache_writing_fixture_free (SoupCacheWritingFixture *fixture)
 	g_slice_free (SoupCacheWritingFixture, fixture);
 }
 
+static void
+msg_content_sniffed_cb (SoupMessage *msg, gchar *content_type, GHashTable *params, SoupCacheWritingFixture *fixture)
+{
+	soup_message_headers_set_content_type (fixture->entry->headers, content_type, params);
+}
+
 static void
 close_ready_cb (GObject *source, GAsyncResult *result, SoupCacheWritingFixture *fixture)
 {
@@ -994,6 +981,8 @@ msg_got_headers_cb (SoupMessage *msg, gpointer user_data)
 
 		/* We connect now to these signals and buffer the data
 		   if it comes before the file is ready for writing */
+		fixture->content_sniffed_handler =
+			g_signal_connect (msg, "content-sniffed", G_CALLBACK (msg_content_sniffed_cb), fixture);
 		fixture->got_chunk_handler =
 			g_signal_connect (msg, "got-chunk", G_CALLBACK (msg_got_chunk_cb), fixture);
 		fixture->got_body_handler =
@@ -1162,7 +1151,6 @@ soup_cache_finalize (GObject *object)
 	g_free (priv->cache_dir);
 
 	g_list_free (priv->lru_start);
-	priv->lru_start = NULL;
 
 	G_OBJECT_CLASS (soup_cache_parent_class)->finalize (object);
 }
@@ -1356,8 +1344,8 @@ soup_cache_has_response (SoupCache *cache, SoupMessage *msg)
 
 	/* 4. The request is a conditional request issued by the client.
 	 */
-	if (soup_message_headers_get (msg->request_headers, "If-Modified-Since") ||
-	    soup_message_headers_get (msg->request_headers, "If-None-Match"))
+	if (soup_message_headers_get_one (msg->request_headers, "If-Modified-Since") ||
+	    soup_message_headers_get_list (msg->request_headers, "If-None-Match"))
 		return SOUP_CACHE_RESPONSE_STALE;
 
 	/* 5. The presented request and stored response are free from
@@ -1368,11 +1356,11 @@ soup_cache_has_response (SoupCache *cache, SoupMessage *msg)
 
 	/* For HTTP 1.0 compatibility. RFC2616 section 14.9.4
 	 */
-	pragma = soup_message_headers_get (msg->request_headers, "Pragma");
+	pragma = soup_message_headers_get_list (msg->request_headers, "Pragma");
 	if (pragma && soup_header_contains (pragma, "no-cache"))
 		return SOUP_CACHE_RESPONSE_STALE;
 
-	cache_control = soup_message_headers_get (msg->request_headers, "Cache-Control");
+	cache_control = soup_message_headers_get_list (msg->request_headers, "Cache-Control");
 	if (cache_control) {
 		GHashTable *hash = soup_header_parse_param_list (cache_control);
 
