@@ -9,11 +9,10 @@
 #include <config.h>
 #endif
 
-#include <stdlib.h>
 #include <string.h>
-#include <gio/gio.h>
 
 #include "soup-filter-input-stream.h"
+#include "soup.h"
 
 /* This is essentially a subset of GDataInputStream, except that we
  * can do the equivalent of "fill_nonblocking()" on it. (We could use
@@ -40,12 +39,11 @@ soup_filter_input_stream_init (SoupFilterInputStream *stream)
 }
 
 static void
-finalize (GObject *object)
+soup_filter_input_stream_finalize (GObject *object)
 {
 	SoupFilterInputStream *fstream = SOUP_FILTER_INPUT_STREAM (object);
 
-	if (fstream->priv->buf)
-		g_byte_array_free (fstream->priv->buf, TRUE);
+	g_clear_pointer (&fstream->priv->buf, g_byte_array_unref);
 
 	G_OBJECT_CLASS (soup_filter_input_stream_parent_class)->finalize (object);
 }
@@ -145,7 +143,7 @@ soup_filter_input_stream_class_init (SoupFilterInputStreamClass *stream_class)
 
 	g_type_class_add_private (stream_class, sizeof (SoupFilterInputStreamPrivate));
 
-	object_class->finalize = finalize;
+	object_class->finalize = soup_filter_input_stream_finalize;
 
 	input_stream_class->read_fn = soup_filter_input_stream_read_fn;
 }
