@@ -9,9 +9,8 @@
 #include <config.h>
 #endif
 
-#include <gio/gio.h>
-
 #include "soup-io-stream.h"
+#include "soup.h"
 #include "soup-filter-input-stream.h"
 
 struct _SoupIOStreamPrivate {
@@ -103,8 +102,7 @@ soup_io_stream_finalize (GObject *object)
 {
 	SoupIOStream *siostream = SOUP_IO_STREAM (object);
 
-	if (siostream->priv->base_iostream)
-		g_object_unref (siostream->priv->base_iostream);
+	g_clear_object (&siostream->priv->base_iostream);
 
 	G_OBJECT_CLASS (soup_io_stream_parent_class)->finalize (object);
 }
