@@ -11,15 +11,11 @@
 
 #define LIBSOUP_I_HAVE_READ_BUG_594377_AND_KNOW_SOUP_PASSWORD_MANAGER_MIGHT_GO_AWAY
 
-#include "soup-address.h"
 #include "soup-session-async.h"
+#include "soup.h"
 #include "soup-session-private.h"
-#include "soup-address.h"
 #include "soup-message-private.h"
 #include "soup-message-queue.h"
-#include "soup-misc.h"
-#include "soup-password-manager.h"
-#include "soup-uri.h"
 
 /**
  * SECTION:soup-session-async
@@ -37,16 +33,6 @@ static void send_request_running   (SoupSession *session, SoupMessageQueueItem *
 static void send_request_restarted (SoupSession *session, SoupMessageQueueItem *item);
 static void send_request_finished  (SoupSession *session, SoupMessageQueueItem *item);
 
-static void  queue_message   (SoupSession *session, SoupMessage *req,
-			      SoupSessionCallback callback, gpointer user_data);
-static guint send_message    (SoupSession *session, SoupMessage *req);
-static void  cancel_message  (SoupSession *session, SoupMessage *msg,
-			      guint status_code);
-static void  kick            (SoupSession *session);
-
-static void  auth_required   (SoupSession *session, SoupMessage *msg,
-			      SoupAuth *auth, gboolean retrying);
-
 G_DEFINE_TYPE (SoupSessionAsync, soup_session_async, SOUP_TYPE_SESSION)
 
 typedef struct {
@@ -65,7 +51,7 @@ soup_session_async_init (SoupSessionAsync *sa)
 }
 
 static void
-dispose (GObject *object)
+soup_session_async_dispose (GObject *object)
 {
 	SoupSessionAsyncPrivate *priv = SOUP_SESSION_ASYNC_GET_PRIVATE (object);
 
@@ -74,25 +60,6 @@ dispose (GObject *object)
 	G_OBJECT_CLASS (soup_session_async_parent_class)->dispose (object);
 }
 
-static void
-soup_session_async_class_init (SoupSessionAsyncClass *soup_session_async_class)
-{
-	SoupSessionClass *session_class = SOUP_SESSION_CLASS (soup_session_async_class);
-	GObjectClass *object_class = G_OBJECT_CLASS (session_class);
-
-	g_type_class_add_private (soup_session_async_class,
-				  sizeof (SoupSessionAsyncPrivate));
-
-	/* virtual method override */
-	session_class->queue_message = queue_message;
-	session_class->send_message = send_message;
-	session_class->cancel_message = cancel_message;
-	session_class->auth_required = auth_required;
-	session_class->kick = kick;
-
-	object_class->dispose = dispose;
-}
-
 
 /**
  * soup_session_async_new:
@@ -338,7 +305,6 @@ process_queue_item (SoupMessageQueueItem *item,
 				item->callback (session, item->msg, item->callback_data);
 			else if (item->new_api)
 				send_request_finished (session, item);
-			g_object_unref (item->msg);
 			do_idle_run_queue (session);
 			g_object_unref (session);
 			return;
@@ -427,25 +393,25 @@ do_idle_run_queue (SoupSession *session)
 }
 
 static void
-queue_message (SoupSession *session, SoupMessage *req,
-	       SoupSessionCallback callback, gpointer user_data)
+soup_session_async_queue_message (SoupSession *session, SoupMessage *req,
+				  SoupSessionCallback callback, gpointer user_data)
 {
-	SOUP_SESSION_CLASS (soup_session_async_parent_class)->queue_message (session, req, callback, user_data);
+	SoupMessageQueueItem *item;
+
+	item = soup_session_append_queue_item (session, req, callback, user_data);
+	soup_message_queue_item_unref (item);
 
 	do_idle_run_queue (session);
 }
 
 static guint
-send_message (SoupSession *session, SoupMessage *req)
+soup_session_async_send_message (SoupSession *session, SoupMessage *req)
 {
 	SoupMessageQueueItem *item;
 	GMainContext *async_context =
 		soup_session_get_async_context (session);
 
-	/* Balance out the unref that queuing will eventually do */
-	g_object_ref (req);
-
-	queue_message (session, req, NULL, NULL);
+	soup_session_async_queue_message (session, req, NULL, NULL);
 
 	item = soup_message_queue_lookup (soup_session_get_queue (session), req);
 	g_return_val_if_fail (item != NULL, SOUP_STATUS_MALFORMED);
@@ -459,8 +425,8 @@ send_message (SoupSession *session, SoupMessage *req)
 }
 
 static void
-cancel_message (SoupSession *session, SoupMessage *msg,
-		guint status_code)
+soup_session_async_cancel_message (SoupSession *session, SoupMessage *msg,
+				   guint status_code)
 {
 	SoupMessageQueue *queue;
 	SoupMessageQueueItem *item;
@@ -502,8 +468,8 @@ got_passwords (SoupPasswordManager *password_manager, SoupMessage *msg,
 }
 
 static void
-auth_required (SoupSession *session, SoupMessage *msg,
-	       SoupAuth *auth, gboolean retrying)
+soup_session_async_auth_required (SoupSession *session, SoupMessage *msg,
+				  SoupAuth *auth, gboolean retrying)
 {
 	SoupSessionFeature *password_manager;
 
@@ -525,7 +491,7 @@ auth_required (SoupSession *session, SoupMessage *msg,
 }
 
 static void
-kick (SoupSession *session)
+soup_session_async_kick (SoupSession *session)
 {
 	do_idle_run_queue (session);
 }
@@ -562,6 +528,7 @@ send_request_restarted (SoupSession *session, SoupMessageQueueItem *item)
 {
 	/* We won't be needing this, then. */
 	g_object_set_data (G_OBJECT (item->msg), "SoupSessionAsync:ostream", NULL);
+	item->io_started = FALSE;
 }
 
 static void
@@ -569,6 +536,7 @@ send_request_finished (SoupSession *session, SoupMessageQueueItem *item)
 {
 	GMemoryOutputStream *mostream;
 	GInputStream *istream = NULL;
+	GError *error = NULL;
 
 	if (!item->result) {
 		/* Something else already took care of it. */
@@ -576,7 +544,7 @@ send_request_finished (SoupSession *session, SoupMessageQueueItem *item)
 	}
 
 	mostream = g_object_get_data (G_OBJECT (item->msg), "SoupSessionAsync:ostream");
-	if (mostream && !SOUP_STATUS_IS_TRANSPORT_ERROR (item->msg->status_code)) {
+	if (mostream) {
 		gpointer data;
 		gssize size;
 
@@ -586,9 +554,21 @@ send_request_finished (SoupSession *session, SoupMessageQueueItem *item)
 		size = g_memory_output_stream_get_data_size (mostream);
 		data = size ? g_memory_output_stream_steal_data (mostream) : g_strdup ("");
 		istream = g_memory_input_stream_new_from_data (data, size, g_free);
+	} else if (item->io_started) {
+		/* The message finished before becoming readable. This
+		 * will happen, eg, if it's cancelled from got-headers.
+		 * Do nothing; the op will complete via read_ready_cb()
+		 * after we return;
+		 */
+		return;
+	} else {
+		/* The message finished before even being started;
+		 * probably a tunnel connect failure.
+		 */
+		istream = g_memory_input_stream_new ();
 	}
 
-	send_request_return_result (item, istream, NULL);
+	send_request_return_result (item, istream, error);
 }
 
 static void
@@ -660,12 +640,6 @@ static gboolean
 read_ready_cb (SoupMessage *msg, gpointer user_data)
 {
 	SoupMessageQueueItem *item = user_data;
-	GError *error = NULL;
-
-	if (g_cancellable_set_error_if_cancelled (item->cancellable, &error)) {
-		send_request_return_result (item, NULL, error);
-		return FALSE;
-	}
 
 	try_run_until_read (item);
 	return FALSE;
@@ -685,6 +659,13 @@ try_run_until_read (SoupMessageQueueItem *item)
 		return;
 	}
 
+	if (g_error_matches (error, SOUP_HTTP_ERROR, SOUP_STATUS_TRY_AGAIN)) {
+		item->state = SOUP_MESSAGE_RESTARTING;
+		soup_message_io_finished (item->msg);
+		g_error_free (error);
+		return;
+	}
+
 	if (!g_error_matches (error, G_IO_ERROR, G_IO_ERROR_WOULD_BLOCK)) {
 		send_request_return_result (item, NULL, error);
 		return;
@@ -700,6 +681,7 @@ try_run_until_read (SoupMessageQueueItem *item)
 static void
 send_request_running (SoupSession *session, SoupMessageQueueItem *item)
 {
+	item->io_started = TRUE;
 	try_run_until_read (item);
 }
 
@@ -720,10 +702,7 @@ soup_session_send_request_async (SoupSession         *session,
 		      NULL);
 	g_return_if_fail (use_thread_context);
 
-	/* Balance out the unref that queuing will eventually do */
-	g_object_ref (msg);
-
-	queue_message (session, msg, NULL, NULL);
+	soup_session_async_queue_message (session, msg, NULL, NULL);
 
 	item = soup_message_queue_lookup (soup_session_get_queue (session), msg);
 	g_return_if_fail (item != NULL);
@@ -755,3 +734,22 @@ soup_session_send_request_finish (SoupSession   *session,
 		return NULL;
 	return g_object_ref (g_simple_async_result_get_op_res_gpointer (simple));
 }
+
+static void
+soup_session_async_class_init (SoupSessionAsyncClass *soup_session_async_class)
+{
+	SoupSessionClass *session_class = SOUP_SESSION_CLASS (soup_session_async_class);
+	GObjectClass *object_class = G_OBJECT_CLASS (session_class);
+
+	g_type_class_add_private (soup_session_async_class,
+				  sizeof (SoupSessionAsyncPrivate));
+
+	/* virtual method override */
+	session_class->queue_message = soup_session_async_queue_message;
+	session_class->send_message = soup_session_async_send_message;
+	session_class->cancel_message = soup_session_async_cancel_message;
+	session_class->auth_required = soup_session_async_auth_required;
+	session_class->kick = soup_session_async_kick;
+
+	object_class->dispose = soup_session_async_dispose;
+}
