@@ -10,6 +10,7 @@
 #endif
 
 #include "soup-session-feature.h"
+#include "soup.h"
 #include "soup-message-private.h"
 
 /**
@@ -42,43 +43,18 @@
  * @request_queued: Proxies the session's #SoupSession::request_queued signal
  * @request_started: Proxies the session's #SoupSession::request_started signal
  * @request_unqueued: Proxies the session's #SoupSession::request_unqueued signal
+ * @add_feature: adds a sub-feature to the main feature
+ * @remove_feature: removes a sub-feature from the main feature
+ * @has_feature: tests if the feature includes a sub-feature
  *
  * The interface implemented by #SoupSessionFeature<!-- -->s.
  *
  * Since: 2.24
  **/
 
-static void soup_session_feature_interface_init (SoupSessionFeatureInterface *interface);
+static void soup_session_feature_default_init (SoupSessionFeatureInterface *interface);
 
-static void attach (SoupSessionFeature *feature, SoupSession *session);
-static void detach (SoupSessionFeature *feature, SoupSession *session);
-
-GType
-soup_session_feature_get_type (void)
-{
-  static volatile gsize g_define_type_id__volatile = 0;
-  if (g_once_init_enter (&g_define_type_id__volatile))
-    {
-      GType g_define_type_id =
-        g_type_register_static_simple (G_TYPE_INTERFACE,
-                                       g_intern_static_string ("SoupSessionFeature"),
-                                       sizeof (SoupSessionFeatureInterface),
-                                       (GClassInitFunc)soup_session_feature_interface_init,
-                                       0,
-                                       (GInstanceInitFunc)NULL,
-                                       (GTypeFlags) 0);
-      g_type_interface_add_prerequisite (g_define_type_id, G_TYPE_OBJECT);
-      g_once_init_leave (&g_define_type_id__volatile, g_define_type_id);
-    }
-  return g_define_type_id__volatile;
-}
-
-static void
-soup_session_feature_interface_init (SoupSessionFeatureInterface *interface)
-{
-	interface->attach = attach;
-	interface->detach = detach;
-}
+G_DEFINE_INTERFACE (SoupSessionFeature, soup_session_feature, G_TYPE_OBJECT)
 
 static void
 weak_notify_unref (gpointer feature, GObject *ex_object)
@@ -118,7 +94,7 @@ request_unqueued (SoupSession *session, SoupMessage *msg, gpointer feature)
 }
 
 static void
-attach (SoupSessionFeature *feature, SoupSession *session)
+soup_session_feature_real_attach (SoupSessionFeature *feature, SoupSession *session)
 {
 	g_object_weak_ref (G_OBJECT (session),
 			   weak_notify_unref, g_object_ref (feature));
@@ -147,7 +123,7 @@ soup_session_feature_attach (SoupSessionFeature *feature,
 }
 
 static void
-detach (SoupSessionFeature *feature, SoupSession *session)
+soup_session_feature_real_detach (SoupSessionFeature *feature, SoupSession *session)
 {
 	g_object_weak_unref (G_OBJECT (session), weak_notify_unref, feature);
 
@@ -165,6 +141,13 @@ soup_session_feature_detach (SoupSessionFeature *feature,
 	SOUP_SESSION_FEATURE_GET_CLASS (feature)->detach (feature, session);
 }
 
+static void
+soup_session_feature_default_init (SoupSessionFeatureInterface *interface)
+{
+	interface->attach = soup_session_feature_real_attach;
+	interface->detach = soup_session_feature_real_detach;
+}
+
 /**
  * soup_session_feature_add_feature:
  * @feature: the "base" feature
