@@ -9,17 +9,13 @@
 #include <config.h>
 #endif
 
-#include <stdio.h>
-#include <errno.h>
-#include <fcntl.h>
-#include <signal.h>
 #include <string.h>
-#include <unistd.h>
 
-#include "soup-address.h"
 #include "soup-socket.h"
+#include "soup.h"
+#include "soup-filter-input-stream.h"
+#include "soup-io-stream.h"
 #include "soup-marshal.h"
-#include "soup-misc.h"
 #include "soup-misc-private.h"
 
 /**
@@ -62,16 +58,17 @@ enum {
 	PROP_CLEAN_DISPOSE,
 	PROP_TLS_CERTIFICATE,
 	PROP_TLS_ERRORS,
+	PROP_USE_PROXY,
 
 	LAST_PROP
 };
 
 typedef struct {
 	SoupAddress *local_addr, *remote_addr;
-	GIOStream *conn;
+	GIOStream *conn, *iostream;
 	GSocket *gsock;
-	GPollableInputStream *istream;
-	GPollableOutputStream *ostream;
+	GInputStream *istream;
+	GOutputStream *ostream;
 	GTlsCertificateFlags tls_errors;
 
 	guint non_blocking:1;
@@ -81,12 +78,12 @@ typedef struct {
 	guint ssl_fallback:1;
 	guint clean_dispose:1;
 	guint use_thread_context:1;
+	guint use_proxy:1;
 	gpointer ssl_creds;
 
 	GMainContext   *async_context;
 	GSource        *watch_src;
 	GSource        *read_src, *write_src;
-	GByteArray     *read_buf;
 
 	GMutex iolock, addrlock;
 	guint timeout;
@@ -95,11 +92,6 @@ typedef struct {
 } SoupSocketPrivate;
 #define SOUP_SOCKET_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), SOUP_TYPE_SOCKET, SoupSocketPrivate))
 
-static void set_property (GObject *object, guint prop_id,
-			  const GValue *value, GParamSpec *pspec);
-static void get_property (GObject *object, guint prop_id,
-			  GValue *value, GParamSpec *pspec);
-
 static void soup_socket_peer_certificate_changed (GObject *conn,
 						  GParamSpec *pspec,
 						  gpointer user_data);
@@ -119,20 +111,9 @@ disconnect_internal (SoupSocket *sock, gboolean close)
 {
 	SoupSocketPrivate *priv = SOUP_SOCKET_GET_PRIVATE (sock);
 
-	if (priv->gsock) {
-		if (close)
-			g_socket_close (priv->gsock, NULL);
-		g_object_unref (priv->gsock);
-		priv->gsock = NULL;
-	}
-	if (priv->conn) {
-		if (G_IS_TLS_CONNECTION (priv->conn))
-			g_signal_handlers_disconnect_by_func (priv->conn, soup_socket_peer_certificate_changed, sock);
-		g_object_unref (priv->conn);
-		priv->conn = NULL;
-		priv->istream = NULL;
-		priv->ostream = NULL;
-	}
+	g_clear_object (&priv->gsock);
+	if (priv->conn && close)
+		g_io_stream_close (priv->conn, NULL, NULL);
 
 	if (priv->read_src) {
 		g_source_destroy (priv->read_src);
@@ -145,7 +126,7 @@ disconnect_internal (SoupSocket *sock, gboolean close)
 }
 
 static void
-finalize (GObject *object)
+soup_socket_finalize (GObject *object)
 {
 	SoupSocketPrivate *priv = SOUP_SOCKET_GET_PRIVATE (object);
 
@@ -154,27 +135,26 @@ finalize (GObject *object)
 			g_warning ("Disposing socket %p during connect", object);
 		g_object_unref (priv->connect_cancel);
 	}
-	if (priv->conn) {
+	if (priv->gsock) {
 		if (priv->clean_dispose)
 			g_warning ("Disposing socket %p while still connected", object);
 		disconnect_internal (SOUP_SOCKET (object), TRUE);
 	}
 
-	if (priv->local_addr)
-		g_object_unref (priv->local_addr);
-	if (priv->remote_addr)
-		g_object_unref (priv->remote_addr);
+	g_clear_object (&priv->conn);
+	g_clear_object (&priv->iostream);
+	g_clear_object (&priv->istream);
+	g_clear_object (&priv->ostream);
+
+	g_clear_object (&priv->local_addr);
+	g_clear_object (&priv->remote_addr);
 
 	if (priv->watch_src) {
 		if (priv->clean_dispose && !priv->is_server)
 			g_warning ("Disposing socket %p during async op", object);
 		g_source_destroy (priv->watch_src);
 	}
-	if (priv->async_context)
-		g_main_context_unref (priv->async_context);
-
-	if (priv->read_buf)
-		g_byte_array_free (priv->read_buf, TRUE);
+	g_clear_pointer (&priv->async_context, g_main_context_unref);
 
 	g_mutex_clear (&priv->addrlock);
 	g_mutex_clear (&priv->iolock);
@@ -182,6 +162,133 @@ finalize (GObject *object)
 	G_OBJECT_CLASS (soup_socket_parent_class)->finalize (object);
 }
 
+
+static void
+finish_socket_setup (SoupSocketPrivate *priv)
+{
+	if (!priv->gsock)
+		return;
+
+	if (!priv->conn)
+		priv->conn = (GIOStream *)g_socket_connection_factory_create_connection (priv->gsock);
+	if (!priv->iostream)
+		priv->iostream = soup_io_stream_new (priv->conn, FALSE);
+	if (!priv->istream)
+		priv->istream = g_object_ref (g_io_stream_get_input_stream (priv->iostream));
+	if (!priv->ostream)
+		priv->ostream = g_object_ref (g_io_stream_get_output_stream (priv->iostream));
+
+	g_socket_set_timeout (priv->gsock, priv->timeout);
+}
+
+static void
+soup_socket_set_property (GObject *object, guint prop_id,
+			  const GValue *value, GParamSpec *pspec)
+{
+	SoupSocketPrivate *priv = SOUP_SOCKET_GET_PRIVATE (object);
+
+	switch (prop_id) {
+	case PROP_LOCAL_ADDRESS:
+		priv->local_addr = (SoupAddress *)g_value_dup_object (value);
+		break;
+	case PROP_REMOTE_ADDRESS:
+		priv->remote_addr = (SoupAddress *)g_value_dup_object (value);
+		break;
+	case PROP_NON_BLOCKING:
+		priv->non_blocking = g_value_get_boolean (value);
+		break;
+	case PROP_SSL_CREDENTIALS:
+		priv->ssl_creds = g_value_get_pointer (value);
+		break;
+	case PROP_SSL_STRICT:
+		priv->ssl_strict = g_value_get_boolean (value);
+		break;
+	case PROP_SSL_FALLBACK:
+		priv->ssl_fallback = g_value_get_boolean (value);
+		break;
+	case PROP_ASYNC_CONTEXT:
+		priv->async_context = g_value_get_pointer (value);
+		if (priv->async_context)
+			g_main_context_ref (priv->async_context);
+		break;
+	case PROP_USE_THREAD_CONTEXT:
+		priv->use_thread_context = g_value_get_boolean (value);
+		break;
+	case PROP_TIMEOUT:
+		priv->timeout = g_value_get_uint (value);
+		if (priv->conn)
+			g_socket_set_timeout (priv->gsock, priv->timeout);
+		break;
+	case PROP_USE_PROXY:
+		priv->use_proxy = g_value_get_boolean (value);
+		break;
+	case PROP_CLEAN_DISPOSE:
+		priv->clean_dispose = g_value_get_boolean (value);
+		break;
+	default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+		break;
+	}
+}
+
+static void
+soup_socket_get_property (GObject *object, guint prop_id,
+			  GValue *value, GParamSpec *pspec)
+{
+	SoupSocketPrivate *priv = SOUP_SOCKET_GET_PRIVATE (object);
+
+	switch (prop_id) {
+	case PROP_LOCAL_ADDRESS:
+		g_value_set_object (value, soup_socket_get_local_address (SOUP_SOCKET (object)));
+		break;
+	case PROP_REMOTE_ADDRESS:
+		g_value_set_object (value, soup_socket_get_remote_address (SOUP_SOCKET (object)));
+		break;
+	case PROP_NON_BLOCKING:
+		g_value_set_boolean (value, priv->non_blocking);
+		break;
+	case PROP_IS_SERVER:
+		g_value_set_boolean (value, priv->is_server);
+		break;
+	case PROP_SSL_CREDENTIALS:
+		g_value_set_pointer (value, priv->ssl_creds);
+		break;
+	case PROP_SSL_STRICT:
+		g_value_set_boolean (value, priv->ssl_strict);
+		break;
+	case PROP_SSL_FALLBACK:
+		g_value_set_boolean (value, priv->ssl_fallback);
+		break;
+	case PROP_TRUSTED_CERTIFICATE:
+		g_value_set_boolean (value, priv->tls_errors == 0);
+		break;
+	case PROP_ASYNC_CONTEXT:
+		g_value_set_pointer (value, priv->async_context ? g_main_context_ref (priv->async_context) : NULL);
+		break;
+	case PROP_USE_THREAD_CONTEXT:
+		g_value_set_boolean (value, priv->use_thread_context);
+		break;
+	case PROP_TIMEOUT:
+		g_value_set_uint (value, priv->timeout);
+		break;
+	case PROP_TLS_CERTIFICATE:
+		if (G_IS_TLS_CONNECTION (priv->conn))
+			g_value_set_object (value, g_tls_connection_get_peer_certificate (G_TLS_CONNECTION (priv->conn)));
+		else
+			g_value_set_object (value, NULL);
+		break;
+	case PROP_TLS_ERRORS:
+		g_value_set_flags (value, priv->tls_errors);
+		break;
+	case PROP_USE_PROXY:
+		g_value_set_boolean (value, priv->use_proxy);
+		break;
+	default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+		break;
+	}
+}
+
 static void
 soup_socket_class_init (SoupSocketClass *socket_class)
 {
@@ -190,9 +297,9 @@ soup_socket_class_init (SoupSocketClass *socket_class)
 	g_type_class_add_private (socket_class, sizeof (SoupSocketPrivate));
 
 	/* virtual method override */
-	object_class->finalize = finalize;
-	object_class->set_property = set_property;
-	object_class->get_property = get_property;
+	object_class->finalize = soup_socket_finalize;
+	object_class->set_property = soup_socket_set_property;
+	object_class->get_property = soup_socket_get_property;
 
 	/* signals */
 
@@ -509,125 +616,14 @@ soup_socket_class_init (SoupSocketClass *socket_class)
 				    "Errors with the peer's TLS certificate",
 				    G_TYPE_TLS_CERTIFICATE_FLAGS, 0,
 				    G_PARAM_READABLE));
-}
 
-
-static void
-finish_socket_setup (SoupSocketPrivate *priv)
-{
-	if (!priv->gsock)
-		return;
-
-	if (!priv->conn)
-		priv->conn = (GIOStream *)g_socket_connection_factory_create_connection (priv->gsock);
-	if (!priv->istream)
-		priv->istream = G_POLLABLE_INPUT_STREAM (g_io_stream_get_input_stream (priv->conn));
-	if (!priv->ostream)
-		priv->ostream = G_POLLABLE_OUTPUT_STREAM (g_io_stream_get_output_stream (priv->conn));
-
-	g_socket_set_timeout (priv->gsock, priv->timeout);
-}
-
-static void
-set_property (GObject *object, guint prop_id,
-	      const GValue *value, GParamSpec *pspec)
-{
-	SoupSocketPrivate *priv = SOUP_SOCKET_GET_PRIVATE (object);
-
-	switch (prop_id) {
-	case PROP_LOCAL_ADDRESS:
-		priv->local_addr = (SoupAddress *)g_value_dup_object (value);
-		break;
-	case PROP_REMOTE_ADDRESS:
-		priv->remote_addr = (SoupAddress *)g_value_dup_object (value);
-		break;
-	case PROP_NON_BLOCKING:
-		priv->non_blocking = g_value_get_boolean (value);
-		break;
-	case PROP_SSL_CREDENTIALS:
-		priv->ssl_creds = g_value_get_pointer (value);
-		break;
-	case PROP_SSL_STRICT:
-		priv->ssl_strict = g_value_get_boolean (value);
-		break;
-	case PROP_SSL_FALLBACK:
-		priv->ssl_fallback = g_value_get_boolean (value);
-		break;
-	case PROP_ASYNC_CONTEXT:
-		priv->async_context = g_value_get_pointer (value);
-		if (priv->async_context)
-			g_main_context_ref (priv->async_context);
-		break;
-	case PROP_USE_THREAD_CONTEXT:
-		priv->use_thread_context = g_value_get_boolean (value);
-		break;
-	case PROP_TIMEOUT:
-		priv->timeout = g_value_get_uint (value);
-		if (priv->conn)
-			g_socket_set_timeout (priv->gsock, priv->timeout);
-		break;
-	case PROP_CLEAN_DISPOSE:
-		priv->clean_dispose = g_value_get_boolean (value);
-		break;
-	default:
-		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-		break;
-	}
-}
-
-static void
-get_property (GObject *object, guint prop_id,
-	      GValue *value, GParamSpec *pspec)
-{
-	SoupSocketPrivate *priv = SOUP_SOCKET_GET_PRIVATE (object);
-
-	switch (prop_id) {
-	case PROP_LOCAL_ADDRESS:
-		g_value_set_object (value, soup_socket_get_local_address (SOUP_SOCKET (object)));
-		break;
-	case PROP_REMOTE_ADDRESS:
-		g_value_set_object (value, soup_socket_get_remote_address (SOUP_SOCKET (object)));
-		break;
-	case PROP_NON_BLOCKING:
-		g_value_set_boolean (value, priv->non_blocking);
-		break;
-	case PROP_IS_SERVER:
-		g_value_set_boolean (value, priv->is_server);
-		break;
-	case PROP_SSL_CREDENTIALS:
-		g_value_set_pointer (value, priv->ssl_creds);
-		break;
-	case PROP_SSL_STRICT:
-		g_value_set_boolean (value, priv->ssl_strict);
-		break;
-	case PROP_SSL_FALLBACK:
-		g_value_set_boolean (value, priv->ssl_fallback);
-		break;
-	case PROP_TRUSTED_CERTIFICATE:
-		g_value_set_boolean (value, priv->tls_errors == 0);
-		break;
-	case PROP_ASYNC_CONTEXT:
-		g_value_set_pointer (value, priv->async_context ? g_main_context_ref (priv->async_context) : NULL);
-		break;
-	case PROP_USE_THREAD_CONTEXT:
-		g_value_set_boolean (value, priv->use_thread_context);
-		break;
-	case PROP_TIMEOUT:
-		g_value_set_uint (value, priv->timeout);
-		break;
-	case PROP_TLS_CERTIFICATE:
-		if (G_IS_TLS_CONNECTION (priv->conn))
-			g_value_set_object (value, g_tls_connection_get_peer_certificate (G_TLS_CONNECTION (priv->conn)));
-		else
-			g_value_set_object (value, NULL);
-		break;
-	case PROP_TLS_ERRORS:
-		g_value_set_flags (value, priv->tls_errors);
-		break;
-	default:
-		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-		break;
-	}
+	g_object_class_install_property (
+		object_class, PROP_USE_PROXY,
+		g_param_spec_boolean (SOUP_SOCKET_USE_PROXY,
+				      "Use proxy",
+				      "Use #GProxyResolver",
+				      FALSE,
+				      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
 }
 
 
@@ -774,6 +770,10 @@ soup_socket_connect_async (SoupSocket *sock, GCancellable *cancellable,
 	client = g_socket_client_new ();
 	g_signal_connect (client, "event",
 			  G_CALLBACK (proxy_socket_client_event), sock);
+	if (priv->use_proxy)
+		g_socket_client_add_application_proxy (client, "http");
+	else
+		g_socket_client_set_enable_proxy (client, FALSE);
 	if (priv->timeout)
 		g_socket_client_set_timeout (client, priv->timeout);
 	g_socket_client_connect_async (client,
@@ -819,6 +819,10 @@ soup_socket_connect_sync (SoupSocket *sock, GCancellable *cancellable)
 	client = g_socket_client_new ();
 	g_signal_connect (client, "event",
 			  G_CALLBACK (proxy_socket_client_event), sock);
+	if (priv->use_proxy)
+		g_socket_client_add_application_proxy (client, "http");
+	else
+		g_socket_client_set_enable_proxy (client, FALSE);
 	if (priv->timeout)
 		g_socket_client_set_timeout (client, priv->timeout);
 	conn = g_socket_client_connect (client,
@@ -846,13 +850,21 @@ soup_socket_get_gsocket (SoupSocket *sock)
 }
 
 GIOStream *
-soup_socket_get_iostream (SoupSocket *sock)
+soup_socket_get_connection (SoupSocket *sock)
 {
 	g_return_val_if_fail (SOUP_IS_SOCKET (sock), NULL);
 
 	return SOUP_SOCKET_GET_PRIVATE (sock)->conn;
 }
 
+GIOStream *
+soup_socket_get_iostream (SoupSocket *sock)
+{
+	g_return_val_if_fail (SOUP_IS_SOCKET (sock), NULL);
+
+	return SOUP_SOCKET_GET_PRIVATE (sock)->iostream;
+}
+
 static GSource *
 soup_socket_create_watch (SoupSocketPrivate *priv, GIOCondition cond,
 			  GPollableSourceFunc callback, gpointer user_data,
@@ -862,9 +874,9 @@ soup_socket_create_watch (SoupSocketPrivate *priv, GIOCondition cond,
 	GMainContext *async_context;
 
 	if (cond == G_IO_IN)
-		watch = g_pollable_input_stream_create_source (priv->istream, cancellable);
+		watch = g_pollable_input_stream_create_source (G_POLLABLE_INPUT_STREAM (priv->istream), cancellable);
 	else
-		watch = g_pollable_output_stream_create_source (priv->ostream, cancellable);
+		watch = g_pollable_output_stream_create_source (G_POLLABLE_OUTPUT_STREAM (priv->ostream), cancellable);
 	g_source_set_callback (watch, (GSourceFunc)callback, user_data, NULL);
 
 	if (priv->use_thread_context)
@@ -1089,8 +1101,13 @@ soup_socket_start_proxy_ssl (SoupSocket *sock, const char *ssl_host,
 	g_signal_connect (priv->conn, "notify::peer-certificate",
 			  G_CALLBACK (soup_socket_peer_certificate_changed), sock);
 
-	priv->istream = G_POLLABLE_INPUT_STREAM (g_io_stream_get_input_stream (priv->conn));
-	priv->ostream = G_POLLABLE_OUTPUT_STREAM (g_io_stream_get_output_stream (priv->conn));
+	g_clear_object (&priv->istream);
+	g_clear_object (&priv->ostream);
+	g_clear_object (&priv->iostream);
+	priv->iostream = soup_io_stream_new (priv->conn, FALSE);
+	priv->istream = g_object_ref (g_io_stream_get_input_stream (priv->iostream));
+	priv->ostream = g_object_ref (g_io_stream_get_output_stream (priv->iostream));
+
 	return TRUE;
 }
 	
@@ -1202,7 +1219,7 @@ soup_socket_disconnect (SoupSocket *sock)
 		g_cancellable_cancel (priv->connect_cancel);
 		return;
 	} else if (g_mutex_trylock (&priv->iolock)) {
-		if (priv->conn)
+		if (priv->gsock)
 			disconnect_internal (sock, TRUE);
 		else
 			already_disconnected = TRUE;
@@ -1252,7 +1269,7 @@ soup_socket_is_connected (SoupSocket *sock)
 	g_return_val_if_fail (SOUP_IS_SOCKET (sock), FALSE);
 	priv = SOUP_SOCKET_GET_PRIVATE (sock);
 
-	return priv->conn != NULL;
+	return priv->conn && !g_io_stream_is_closed (priv->conn);
 }
 
 /**
@@ -1321,6 +1338,25 @@ soup_socket_get_remote_address (SoupSocket *sock)
 	return priv->remote_addr;
 }
 
+SoupURI *
+soup_socket_get_http_proxy_uri (SoupSocket *sock)
+{
+	SoupSocketPrivate *priv = SOUP_SOCKET_GET_PRIVATE (sock);
+	GSocketAddress *addr;
+	GProxyAddress *paddr;
+
+	if (!priv->gsock)
+		return NULL;
+	addr = g_socket_get_remote_address (priv->gsock, NULL);
+	if (!addr || !G_IS_PROXY_ADDRESS (addr))
+		return NULL;
+
+	paddr = G_PROXY_ADDRESS (addr);
+	if (strcmp (g_proxy_address_get_protocol (paddr), "http") != 0)
+		return NULL;
+
+	return soup_uri_new (g_proxy_address_get_uri (paddr));
+}
 
 static gboolean
 socket_read_watch (GObject *pollable, gpointer user_data)
@@ -1334,34 +1370,18 @@ socket_read_watch (GObject *pollable, gpointer user_data)
 }
 
 static SoupSocketIOStatus
-read_from_network (SoupSocket *sock, gpointer buffer, gsize len,
-		   gsize *nread, GCancellable *cancellable, GError **error)
+translate_read_status (SoupSocket *sock, GCancellable *cancellable,
+		       gssize my_nread, gsize *nread,
+		       GError *my_err, GError **error)
 {
 	SoupSocketPrivate *priv = SOUP_SOCKET_GET_PRIVATE (sock);
-	GError *my_err = NULL;
-	gssize my_nread;
-
-	*nread = 0;
-
-	if (!priv->conn)
-		return SOUP_SOCKET_EOF;
-
-	if (!priv->non_blocking) {
-		my_nread = g_input_stream_read (G_INPUT_STREAM (priv->istream),
-						buffer, len,
-						cancellable, &my_err);
-	} else {
-		my_nread = g_pollable_input_stream_read_nonblocking (
-			priv->istream, buffer, len,
-			cancellable, &my_err);
-	}
 
 	if (my_nread > 0) {
-		g_clear_error (&my_err);
+		g_assert_no_error (my_err);
 		*nread = my_nread;
 		return SOUP_SOCKET_OK;
 	} else if (my_nread == 0) {
-		g_clear_error (&my_err);
+		g_assert_no_error (my_err);
 		*nread = my_nread;
 		return SOUP_SOCKET_EOF;
 	} else if (g_error_matches (my_err, G_IO_ERROR, G_IO_ERROR_WOULD_BLOCK)) {
@@ -1379,27 +1399,6 @@ read_from_network (SoupSocket *sock, gpointer buffer, gsize len,
 	return SOUP_SOCKET_ERROR;
 }
 
-static SoupSocketIOStatus
-read_from_buf (SoupSocket *sock, gpointer buffer, gsize len, gsize *nread)
-{
-	SoupSocketPrivate *priv = SOUP_SOCKET_GET_PRIVATE (sock);
-	GByteArray *read_buf = priv->read_buf;
-
-	*nread = MIN (read_buf->len, len);
-	memcpy (buffer, read_buf->data, *nread);
-
-	if (*nread == read_buf->len) {
-		g_byte_array_free (read_buf, TRUE);
-		priv->read_buf = NULL;
-	} else {
-		memmove (read_buf->data, read_buf->data + *nread, 
-			 read_buf->len - *nread);
-		g_byte_array_set_size (read_buf, read_buf->len - *nread);
-	}
-
-	return SOUP_SOCKET_OK;
-}
-
 /**
  * SoupSocketIOStatus:
  * @SOUP_SOCKET_OK: Success
@@ -1443,6 +1442,8 @@ soup_socket_read (SoupSocket *sock, gpointer buffer, gsize len,
 {
 	SoupSocketPrivate *priv;
 	SoupSocketIOStatus status;
+	gssize my_nread;
+	GError *my_err = NULL;
 
 	g_return_val_if_fail (SOUP_IS_SOCKET (sock), SOUP_SOCKET_ERROR);
 	g_return_val_if_fail (nread != NULL, SOUP_SOCKET_ERROR);
@@ -1450,10 +1451,24 @@ soup_socket_read (SoupSocket *sock, gpointer buffer, gsize len,
 	priv = SOUP_SOCKET_GET_PRIVATE (sock);
 
 	g_mutex_lock (&priv->iolock);
-	if (priv->read_buf)
-		status = read_from_buf (sock, buffer, len, nread);
-	else
-		status = read_from_network (sock, buffer, len, nread, cancellable, error);
+
+	if (!priv->istream) {
+		status = SOUP_SOCKET_EOF;
+		goto out;
+	}
+
+	if (!priv->non_blocking) {
+		my_nread = g_input_stream_read (priv->istream, buffer, len,
+						cancellable, &my_err);
+	} else {
+		my_nread = g_pollable_input_stream_read_nonblocking (G_POLLABLE_INPUT_STREAM (priv->istream),
+								     buffer, len,
+								     cancellable, &my_err);
+	}
+	status = translate_read_status (sock, cancellable,
+					my_nread, nread, my_err, error);
+
+out:
 	g_mutex_unlock (&priv->iolock);
 
 	return status;
@@ -1495,9 +1510,8 @@ soup_socket_read_until (SoupSocket *sock, gpointer buffer, gsize len,
 {
 	SoupSocketPrivate *priv;
 	SoupSocketIOStatus status;
-	GByteArray *read_buf;
-	guint match_len, prev_len;
-	guint8 *p, *end;
+	gssize my_nread;
+	GError *my_err = NULL;
 
 	g_return_val_if_fail (SOUP_IS_SOCKET (sock), SOUP_SOCKET_ERROR);
 	g_return_val_if_fail (nread != NULL, SOUP_SOCKET_ERROR);
@@ -1509,41 +1523,18 @@ soup_socket_read_until (SoupSocket *sock, gpointer buffer, gsize len,
 
 	*got_boundary = FALSE;
 
-	if (!priv->read_buf)
-		priv->read_buf = g_byte_array_new ();
-	read_buf = priv->read_buf;
-
-	if (read_buf->len < boundary_len) {
-		prev_len = read_buf->len;
-		g_byte_array_set_size (read_buf, len);
-		status = read_from_network (sock,
-					    read_buf->data + prev_len,
-					    len - prev_len, nread, cancellable, error);
-		read_buf->len = prev_len + *nread;
-
-		if (status != SOUP_SOCKET_OK) {
-			g_mutex_unlock (&priv->iolock);
-			return status;
-		}
+	if (!priv->istream)
+		status = SOUP_SOCKET_EOF;
+	else {
+		my_nread = soup_filter_input_stream_read_until (
+			SOUP_FILTER_INPUT_STREAM (priv->istream),
+			buffer, len, boundary, boundary_len,
+			!priv->non_blocking,
+			got_boundary, cancellable, &my_err);
+		status = translate_read_status (sock, cancellable,
+						my_nread, nread, my_err, error);
 	}
 
-	/* Scan for the boundary */
-	end = read_buf->data + read_buf->len;
-	for (p = read_buf->data; p <= end - boundary_len; p++) {
-		if (!memcmp (p, boundary, boundary_len)) {
-			p += boundary_len;
-			*got_boundary = TRUE;
-			break;
-		}
-	}
-
-	/* Return everything up to 'p' (which is either just after the
-	 * boundary, or @boundary_len - 1 bytes before the end of the
-	 * buffer).
-	 */
-	match_len = p - read_buf->data;
-	status = read_from_buf (sock, buffer, MIN (len, match_len), nread);
-
 	g_mutex_unlock (&priv->iolock);
 	return status;
 }
@@ -1611,13 +1602,13 @@ soup_socket_write (SoupSocket *sock, gconstpointer buffer,
 	}
 
 	if (!priv->non_blocking) {
-		my_nwrote = g_output_stream_write (G_OUTPUT_STREAM (priv->ostream),
+		my_nwrote = g_output_stream_write (priv->ostream,
 						   buffer, len,
 						   cancellable, &my_err);
 	} else {
 		my_nwrote = g_pollable_output_stream_write_nonblocking (
-			priv->ostream, buffer, len,
-			cancellable, &my_err);
+			G_POLLABLE_OUTPUT_STREAM (priv->ostream),
+			buffer, len, cancellable, &my_err);
 	}
 
 	if (my_nwrote > 0) {
